@@ -31,7 +31,7 @@ class QuitScreen(ModalScreen):
 
 
 class ModalApp(App):
-    """A app with a modal dialog."""
+    """An app with a modal dialog."""
 
     CSS_PATH = "modal01.css"
     BINDINGS = [("q", "request_quit", "Quit")]
@@ -42,6 +42,7 @@ class ModalApp(App):
         yield Footer()
 
     def action_request_quit(self) -> None:
+        """Action to display the quit dialog."""
         self.push_screen(QuitScreen())
 
 
