@@ -58,6 +58,7 @@ def work(
     group: str = "default",
     exit_on_error: bool = True,
     exclusive: bool = False,
+    description: str | None = None,
 ) -> Callable[FactoryParamSpec, Worker[ReturnType]] | Decorator:
     """A decorator used to create [workers](/guide/workers).
 
@@ -67,6 +68,9 @@ def work(
         group: A short string to identify a group of workers.
         exit_on_error: Exit the app if the worker raises an error. Set to `False` to suppress exceptions.
         exclusive: Cancel all workers in the same group.
+        description: Readable description of the worker for debugging purposes.
+            By default, it uses a string representation of the decorated method
+            and its arguments.
     """
 
     def decorator(
@@ -87,21 +91,25 @@ def work(
             self = args[0]
             assert isinstance(self, DOMNode)
 
-            positional_arguments = ", ".join(repr(arg) for arg in args[1:])
-            keyword_arguments = ", ".join(
-                f"{name}={value!r}" for name, value in kwargs.items()
-            )
-            tokens = [positional_arguments, keyword_arguments]
-            worker_description = (
-                f"{method.__name__}({', '.join(token for token in tokens if token)})"
-            )
+            if description is not None:
+                debug_description = description
+            else:
+                try:
+                    positional_arguments = ", ".join(repr(arg) for arg in args[1:])
+                    keyword_arguments = ", ".join(
+                        f"{name}={value!r}" for name, value in kwargs.items()
+                    )
+                    tokens = [positional_arguments, keyword_arguments]
+                    debug_description = f"{method.__name__}({', '.join(token for token in tokens if token)})"
+                except Exception:
+                    debug_description = "<worker>"
             worker = cast(
                 "Worker[ReturnType]",
                 self.run_worker(
                     partial(method, *args, **kwargs),
                     name=name or method.__name__,
                     group=group,
-                    description=worker_description,
+                    description=debug_description,
                     exclusive=exclusive,
                     exit_on_error=exit_on_error,
                 ),
