@@ -63,12 +63,14 @@ VALID_STYLE_FLAGS: Final = {
 }
 VALID_PSEUDO_CLASSES: Final = {
     "blur",
+    "can-focus",
     "disabled",
     "enabled",
     "focus-within",
     "focus",
     "hover",
 }
-
+VALID_OVERLAY: Final = {"none", "screen"}
+VALID_CONSTRAIN: Final = {"x", "y", "both", "inflect", "none"}
 
 NULL_SPACING: Final = Spacing.all(0)
