@@ -15,7 +15,6 @@ from rich.style import Style
 from rich.syntax import Syntax
 from rich.text import Text
 
-from .. import messages
 from ..dom import DOMNode
 from ..widget import Widget
 from .errors import StylesheetError
@@ -263,6 +262,14 @@ class Stylesheet:
         for path in paths:
             self.read(path)
 
+    def has_source(self, path: str | PurePath) -> bool:
+        """Check if the stylesheet has this CSS source already.
+
+        Returns:
+            Whether the stylesheet is aware of this CSS source or not.
+        """
+        return str(path) in self.source
+
     def add_source(
         self,
         css: str,
@@ -275,7 +282,6 @@ class Stylesheet:
         Args:
             css: String with CSS source.
             path: The path of the source if a file, or some other identifier.
-                Defaults to None.
             is_default_css: True if the CSS is defined in the Widget, False if the CSS is defined
                 in a user stylesheet.
             tie_breaker: Integer representing the priority of this source.
@@ -334,7 +340,6 @@ class Stylesheet:
         Raises:
             StylesheetError: If the CSS could not be read.
             StylesheetParseError: If the CSS is invalid.
-
         """
         # Do this in a fresh Stylesheet so if there are errors we don't break self.
         stylesheet = Stylesheet(variables=self._variables)
@@ -365,12 +370,12 @@ class Stylesheet:
         """Apply the stylesheet to a DOM node.
 
         Args:
-            node: The ``DOMNode`` to apply the stylesheet to.
-                Applies the styles defined in this ``Stylesheet`` to the node.
+            node: The `DOMNode` to apply the stylesheet to.
+                Applies the styles defined in this `Stylesheet` to the node.
                 If the same rule is defined multiple times for the node (e.g. multiple
                 classes modifying the same CSS property), then only the most specific
                 rule will be applied.
-            animate: Animate changed rules. Defaults to ``False``.
+            animate: Animate changed rules.
         """
         # Dictionary of rule attribute names e.g. "text_background" to list of tuples.
         # The tuples contain the rule specificity, and the value for that rule.
@@ -447,7 +452,7 @@ class Stylesheet:
         Args:
             node: A DOM node.
             rules: Mapping of rules.
-            animate: Enable animation. Defaults to False.
+            animate: Enable animation.
         """
 
         # Alias styles and base styles
@@ -519,7 +524,7 @@ class Stylesheet:
 
         Args:
             root: Root note to update.
-            animate: Enable CSS animation. Defaults to False.
+            animate: Enable CSS animation.
         """
 
         self.update_nodes(root.walk_children(with_self=True), animate=animate)
@@ -529,9 +534,8 @@ class Stylesheet:
 
         Args:
             nodes: Nodes to update.
-            animate: Enable CSS animation. Defaults to False.
+            animate: Enable CSS animation.
         """
-
         rules_map = self.rules_map
         apply = self.apply
 
