@@ -240,7 +240,7 @@ class DOMNode(MessagePump):
         description: str = "",
         exit_on_error: bool = True,
         start: bool = True,
-        exclusive: bool = True,
+        exclusive: bool = False,
     ) -> Worker[ResultType]:
         """Run work in a worker.
 
@@ -418,7 +418,7 @@ class DOMNode(MessagePump):
             """Get a path to the DOM Node"""
             try:
                 return f"{getfile(base)}:{base.__name__}"
-            except TypeError:
+            except (TypeError, OSError):
                 return f"{base.__name__}"
 
         for tie_breaker, base in enumerate(self._node_bases):
@@ -457,7 +457,7 @@ class DOMNode(MessagePump):
         return cast("DOMNode | None", self._parent)
 
     @property
-    def screen(self) -> "Screen":
+    def screen(self) -> "Screen[object]":
         """The screen containing this node.
 
         Returns:
@@ -744,16 +744,24 @@ class DOMNode(MessagePump):
         """
         background = Color(0, 0, 0, 0)
         color = Color(255, 255, 255, 0)
+
         style = Style()
+        opacity = 1.0
+
         for node in reversed(self.ancestors_with_self):
             styles = node.styles
+            opacity *= styles.opacity
             if styles.has_rule("background"):
-                background += styles.background
+                text_background = background + styles.background
+                background += styles.background.multiply_alpha(opacity)
+            else:
+                text_background = background
             if styles.has_rule("color"):
                 color = styles.color
             style += styles.text_style
             if styles.has_rule("auto_color") and styles.auto_color:
-                color = background.get_contrast_text(color.a)
+                color = text_background.get_contrast_text(color.a)
+
         style += Style.from_color(
             (background + color).rich_color if (background.a or color.a) else None,
             background.rich_color if background.a else None,
@@ -818,6 +826,22 @@ class DOMNode(MessagePump):
             background += styles.background
         return (base_background, background)
 
+    @property
+    def _opacity_background_colors(self) -> tuple[Color, Color]:
+        """Background colors adjusted for opacity.
+
+        Returns:
+            `(<background color>, <color>)`
+        """
+        base_background = background = BLACK
+        opacity = 1.0
+        for node in reversed(self.ancestors_with_self):
+            styles = node.styles
+            base_background = background
+            opacity *= styles.opacity
+            background += styles.background.multiply_alpha(opacity)
+        return (base_background, background)
+
     @property
     def colors(self) -> tuple[Color, Color, Color, Color]:
         """The widget's background and foreground colors, and the parent's background and foreground colors.
@@ -887,7 +911,7 @@ class DOMNode(MessagePump):
 
         Example:
 
-            Here's how you could detect when the app changes from dark to light mode (and visa versa).
+            Here's how you could detect when the app changes from dark to light mode (and vice versa).
 
             ```python
             def on_dark_change(old_value:bool, new_value:bool):
@@ -1058,6 +1082,7 @@ class DOMNode(MessagePump):
         Returns:
             A widget matching the selector.
         """
+        _rich_traceback_omit = True
         from .css.query import DOMQuery
 
         if isinstance(selector, str):
@@ -1122,7 +1147,8 @@ class DOMNode(MessagePump):
         """Replace all classes.
 
         Args:
-            A string contain space separated classes, or an iterable of class names.
+            classes: A string containing space separated classes, or an
+                iterable of class names.
 
         Returns:
             Self.
