@@ -35,9 +35,6 @@ if TYPE_CHECKING:
 class Event(Message):
     """The base class for all events."""
 
-    def __rich_repr__(self) -> rich.repr.Result:
-        yield from ()
-
 
 @rich.repr.auto
 class Callback(Event, bubble=False, verbose=True):
@@ -141,7 +138,7 @@ class Mount(Event, bubble=False, verbose=False):
     """
 
 
-class Unmount(Mount, bubble=False, verbose=False):
+class Unmount(Event, bubble=False, verbose=False):
     """Sent when a widget is unmounted and may not longer receive messages.
 
     - [ ] Bubbles
