@@ -10,31 +10,34 @@ import threading
 from asyncio import CancelledError, Queue, QueueEmpty, Task
 from contextlib import contextmanager
 from functools import partial
-from typing import TYPE_CHECKING, Any, Awaitable, Callable, Generator, Iterable
+from typing import TYPE_CHECKING, Any, Awaitable, Callable, Generator, Iterable, cast
 from weakref import WeakSet
 
 from . import Logger, events, log, messages
 from ._asyncio import create_task
 from ._callback import invoke
-from ._context import (
-    NoActiveAppError,
-    active_app,
-    active_message_pump,
-    prevent_message_types_stack,
-)
+from ._context import NoActiveAppError, active_app, active_message_pump
+from ._context import message_hook as message_hook_context_var
+from ._context import prevent_message_types_stack
+from ._on import OnNoWidget
 from ._time import time
 from ._types import CallbackType
 from .case import camel_to_snake
 from .css.match import match
-from .css.parse import parse_selectors
 from .errors import DuplicateKeyHandlers
 from .events import Event
 from .message import Message
-from .reactive import Reactive
+from .reactive import Reactive, TooManyComputesError
 from .timer import Timer, TimerCallback
 
 if TYPE_CHECKING:
+    from typing_extensions import TypeAlias
+
     from .app import App
+    from .css.model import SelectorSet
+
+
+Callback: TypeAlias = "Callable[..., Any] | Callable[..., Awaitable[Any]]"
 
 
 class CallbackError(Exception):
@@ -60,18 +63,41 @@ class _MessagePumpMeta(type):
         namespace = camel_to_snake(name)
         isclass = inspect.isclass
         handlers: dict[
-            type[Message], list[tuple[Callable, str | None]]
+            type[Message], list[tuple[Callable, dict[str, tuple[SelectorSet, ...]]]]
         ] = class_dict.get("_decorated_handlers", {})
 
         class_dict["_decorated_handlers"] = handlers
 
         for value in class_dict.values():
             if callable(value) and hasattr(value, "_textual_on"):
-                for message_type, selector in getattr(value, "_textual_on"):
-                    handlers.setdefault(message_type, []).append((value, selector))
+                textual_on: list[
+                    tuple[type[Message], dict[str, tuple[SelectorSet, ...]]]
+                ] = getattr(value, "_textual_on")
+                for message_type, selectors in textual_on:
+                    handlers.setdefault(message_type, []).append((value, selectors))
             if isclass(value) and issubclass(value, Message):
-                if "namespace" not in value.__dict__:
-                    value.namespace = namespace
+                if "namespace" in value.__dict__:
+                    value.handler_name = f"on_{value.__dict__['namespace']}_{camel_to_snake(value.__name__)}"
+                else:
+                    value.handler_name = (
+                        f"on_{namespace}_{camel_to_snake(value.__name__)}"
+                    )
+
+        # Look for reactives with public AND private compute methods.
+        prefix = "compute_"
+        prefix_len = len(prefix)
+        for attr_name, value in class_dict.items():
+            if attr_name.startswith(prefix) and callable(value):
+                reactive_name = attr_name[prefix_len:]
+                if (
+                    reactive_name in class_dict
+                    and isinstance(class_dict[reactive_name], Reactive)
+                    and f"_{attr_name}" in class_dict
+                ):
+                    raise TooManyComputesError(
+                        f"reactive {reactive_name!r} can't have two computes."
+                    )
+
         class_obj = super().__new__(cls, name, bases, class_dict, **kwargs)
         return class_obj
 
@@ -153,7 +179,7 @@ class MessagePump(metaclass=_MessagePumpMeta):
         return self._parent is not None
 
     @property
-    def app(self) -> "App":
+    def app(self) -> "App[object]":
         """
         Get the current app.
 
@@ -348,20 +374,25 @@ class MessagePump(metaclass=_MessagePumpMeta):
         self._timers.add(timer)
         return timer
 
-    def call_after_refresh(self, callback: Callable, *args: Any, **kwargs: Any) -> None:
+    def call_after_refresh(self, callback: Callback, *args: Any, **kwargs: Any) -> bool:
         """Schedule a callback to run after all messages are processed and the screen
         has been refreshed. Positional and keyword arguments are passed to the callable.
 
         Args:
             callback: A callable.
+
+        Returns:
+            `True` if the callback was scheduled, or `False` if the callback could not be
+                scheduled (may occur if the message pump was closed or closing).
+
         """
         # We send the InvokeLater message to ourselves first, to ensure we've cleared
         # out anything already pending in our own queue.
 
         message = messages.InvokeLater(partial(callback, *args, **kwargs))
-        self.post_message(message)
+        return self.post_message(message)
 
-    def call_later(self, callback: Callable, *args: Any, **kwargs: Any) -> None:
+    def call_later(self, callback: Callback, *args: Any, **kwargs: Any) -> bool:
         """Schedule a callback to run after all messages are processed in this object.
         Positional and keywords arguments are passed to the callable.
 
@@ -369,11 +400,16 @@ class MessagePump(metaclass=_MessagePumpMeta):
             callback: Callable to call next.
             *args: Positional arguments to pass to the callable.
             **kwargs: Keyword arguments to pass to the callable.
+
+        Returns:
+            `True` if the callback was scheduled, or `False` if the callback could not be
+                scheduled (may occur if the message pump was closed or closing).
+
         """
         message = events.Callback(callback=partial(callback, *args, **kwargs))
-        self.post_message(message)
+        return self.post_message(message)
 
-    def call_next(self, callback: Callable, *args: Any, **kwargs: Any) -> None:
+    def call_next(self, callback: Callback, *args: Any, **kwargs: Any) -> None:
         """Schedule a callback to run immediately after processing the current message.
 
         Args:
@@ -386,7 +422,9 @@ class MessagePump(metaclass=_MessagePumpMeta):
 
     def _on_invoke_later(self, message: messages.InvokeLater) -> None:
         # Forward InvokeLater message to the Screen
-        self.app.screen._invoke_later(message.callback)
+        self.app.screen._invoke_later(
+            message.callback, message._sender or active_message_pump.get()
+        )
 
     def _close_messages_no_wait(self) -> None:
         """Request the message queue to immediately exit."""
@@ -463,6 +501,7 @@ class MessagePump(metaclass=_MessagePumpMeta):
         """Process messages until the queue is closed."""
         _rich_traceback_guard = True
         self._thread_id = threading.get_ident()
+
         while not self._closed:
             try:
                 message = await self._get_message()
@@ -538,6 +577,13 @@ class MessagePump(metaclass=_MessagePumpMeta):
         if message.no_dispatch:
             return
 
+        try:
+            message_hook = message_hook_context_var.get()
+        except LookupError:
+            pass
+        else:
+            message_hook(message)
+
         with self.prevent(*message._prevent):
             # Allow apps to treat events and messages separately
             if isinstance(message, Event):
@@ -556,22 +602,45 @@ class MessagePump(metaclass=_MessagePumpMeta):
             method_name: Handler method name.
             message: Message object.
         """
+        from .widget import Widget
+
+        methods_dispatched: set[Callable] = set()
+        message_mro = [
+            _type for _type in message.__class__.__mro__ if issubclass(_type, Message)
+        ]
         for cls in self.__class__.__mro__:
             if message._no_default_action:
                 break
             # Try decorated handlers first
-            decorated_handlers = cls.__dict__.get("_decorated_handlers")
-            if decorated_handlers is not None:
-                handlers = decorated_handlers.get(type(message), [])
-                for method, selector in handlers:
-                    if selector is None:
-                        yield cls, method.__get__(self, cls)
-                    else:
-                        selector_sets = parse_selectors(selector)
-                        if message._sender is not None and match(
-                            selector_sets, message.control
-                        ):
+            decorated_handlers = cast(
+                "dict[type[Message], list[tuple[Callable, dict[str, tuple[SelectorSet, ...]]]]] | None",
+                cls.__dict__.get("_decorated_handlers"),
+            )
+
+            if decorated_handlers:
+                for message_class in message_mro:
+                    handlers = decorated_handlers.get(message_class, [])
+
+                    for method, selectors in handlers:
+                        if method in methods_dispatched:
+                            continue
+                        if not selectors:
                             yield cls, method.__get__(self, cls)
+                            methods_dispatched.add(method)
+                        else:
+                            if not message._sender:
+                                continue
+                            for attribute, selector in selectors.items():
+                                node = getattr(message, attribute)
+                                if not isinstance(node, Widget):
+                                    raise OnNoWidget(
+                                        f"on decorator can't match against {attribute!r} as it is not a widget."
+                                    )
+                                if not match(selector, node):
+                                    break
+                            else:
+                                yield cls, method.__get__(self, cls)
+                                methods_dispatched.add(method)
 
             # Fall back to the naming convention
             # But avoid calling the handler if it was decorated
@@ -596,7 +665,7 @@ class MessagePump(metaclass=_MessagePumpMeta):
             message: A Message object.
         """
         _rich_traceback_guard = True
-        handler_name = message._handler_name
+        handler_name = message.handler_name
 
         # Look through the MRO to find a handler
         dispatched = False
@@ -648,6 +717,7 @@ class MessagePump(metaclass=_MessagePumpMeta):
         Returns:
             `True` if the messages was processed, `False` if it wasn't.
         """
+        _rich_traceback_omit = True
         if self._closing or self._closed:
             return False
         if not self.check_message_enabled(message):
@@ -709,7 +779,7 @@ class MessagePump(metaclass=_MessagePumpMeta):
                     )
                 # If key handlers return False, then they are not considered handled
                 # This allows key handlers to do some conditional logic
-                handled = (await invoke(key_method, event)) != False
+                handled = (await invoke(key_method, event)) is not False
                 invoked_method = key_method
 
         return handled
