@@ -5,10 +5,12 @@ The `Screen` class is a special widget which represents the content in the termi
 
 from __future__ import annotations
 
+from functools import partial
 from typing import (
     TYPE_CHECKING,
     Awaitable,
     Callable,
+    ClassVar,
     Generic,
     Iterable,
     Iterator,
@@ -25,25 +27,30 @@ from rich.style import Style
 from . import errors, events, messages
 from ._callback import invoke
 from ._compositor import Compositor, MapGeometry
-from ._context import visible_screen_stack
+from ._context import active_message_pump, visible_screen_stack
+from ._path import CSSPathType, _css_path_type_as_list, _make_path_object_relative
 from ._types import CallbackType
 from .binding import Binding
 from .css.match import match
 from .css.parse import parse_selectors
-from .css.query import QueryType
+from .css.query import NoMatches, QueryType
 from .dom import DOMNode
 from .geometry import Offset, Region, Size
+from .notifications import Notification, SeverityLevel
 from .reactive import Reactive
 from .renderables.background_screen import BackgroundScreen
 from .renderables.blank import Blank
 from .timer import Timer
 from .widget import Widget
+from .widgets import Tooltip
+from .widgets._toast import ToastRack
 
 if TYPE_CHECKING:
     from typing_extensions import Final
 
     # Unused & ignored imports are needed for the docs to link to these objects:
     from .errors import NoWidget  # type: ignore  # noqa: F401
+    from .message_pump import MessagePump
 
 # Screen updates will be batched so that they don't happen more often than 60 times per second:
 UPDATE_PERIOD: Final[float] = 1 / 60
@@ -93,6 +100,27 @@ class ResultCallback(Generic[ScreenResultType]):
 class Screen(Generic[ScreenResultType], Widget):
     """The base class for screens."""
 
+    AUTO_FOCUS: ClassVar[str | None] = None
+    """A selector to determine what to focus automatically when the screen is activated.
+
+    The widget focused is the first that matches the given [CSS selector](/guide/queries/#query-selectors).
+    Set to `None` to inherit the value from the screen's app.
+    Set to `""` to disable auto focus.
+    """
+
+    CSS: ClassVar[str] = ""
+    """Inline CSS, useful for quick scripts. Rules here take priority over CSS_PATH.
+
+    Note:
+        This CSS applies to the whole app.
+    """
+    CSS_PATH: ClassVar[CSSPathType | None] = None
+    """File paths to load CSS from.
+
+    Note:
+        This CSS applies to the whole app.
+    """
+
     DEFAULT_CSS = """
     Screen {
         layout: vertical;
@@ -100,7 +128,6 @@ class Screen(Generic[ScreenResultType], Widget):
         background: $surface;
     }
     """
-
     focused: Reactive[Widget | None] = Reactive(None)
     """The focused [widget][textual.widget.Widget] or `None` for no focus."""
     stack_updates: Reactive[int] = Reactive(0, repaint=False)
@@ -130,18 +157,27 @@ class Screen(Generic[ScreenResultType], Widget):
         self._compositor = Compositor()
         self._dirty_widgets: set[Widget] = set()
         self.__update_timer: Timer | None = None
-        self._callbacks: list[CallbackType] = []
+        self._callbacks: list[tuple[CallbackType, MessagePump]] = []
         self._result_callbacks: list[ResultCallback[ScreenResultType]] = []
 
+        self._tooltip_widget: Widget | None = None
+        self._tooltip_timer: Timer | None = None
+
+        css_paths = [
+            _make_path_object_relative(css_path, self)
+            for css_path in (
+                _css_path_type_as_list(self.CSS_PATH)
+                if self.CSS_PATH is not None
+                else []
+            )
+        ]
+        self.css_path = css_paths
+
     @property
     def is_modal(self) -> bool:
         """Is the screen modal?"""
         return self._modal
 
-    @property
-    def is_transparent(self) -> bool:
-        return False
-
     @property
     def is_current(self) -> bool:
         """Is the screen current (i.e. visible to user)?"""
@@ -161,6 +197,20 @@ class Screen(Generic[ScreenResultType], Widget):
             )
         return self.__update_timer
 
+    @property
+    def layers(self) -> tuple[str, ...]:
+        """Layers from parent.
+
+        Returns:
+            Tuple of layer names.
+        """
+        extras = []
+        if not self.app._disable_notifications:
+            extras.append("_toastrack")
+        if not self.app._disable_tooltips:
+            extras.append("_tooltips")
+        return (*super().layers, *extras)
+
     def render(self) -> RenderableType:
         background = self.styles.background
         try:
@@ -168,7 +218,7 @@ class Screen(Generic[ScreenResultType], Widget):
         except IndexError:
             base_screen = None
 
-        if base_screen is not None and 1 > background.a > 0:
+        if base_screen is not None and background.a < 1:
             return BackgroundScreen(base_screen, background)
 
         if background.is_transparent:
@@ -218,7 +268,7 @@ class Screen(Generic[ScreenResultType], Widget):
             y: Y Coordinate.
 
         Returns:
-            Rich Style object
+            Rich Style object.
         """
         return self._compositor.get_style_at(x, y)
 
@@ -239,6 +289,9 @@ class Screen(Generic[ScreenResultType], Widget):
     @property
     def focus_chain(self) -> list[Widget]:
         """A list of widgets that may receive focus, in focus order."""
+        # TODO: Calculating a focus chain is moderately expensive.
+        # Suspect we can move focus without calculating the entire thing again.
+
         widgets: list[Widget] = []
         add_widget = widgets.append
         stack: list[Iterator[Widget]] = [iter(self.focusable_children)]
@@ -276,6 +329,8 @@ class Screen(Generic[ScreenResultType], Widget):
                 is not `None`, then it is guaranteed that the widget returned matches
                 the CSS selectors given in the argument.
         """
+        # TODO: This shouldn't be required
+        self._compositor._full_map_invalidated = True
         if not isinstance(selector, str):
             selector = selector.__name__
         selector_set = parse_selectors(selector)
@@ -374,6 +429,7 @@ class Screen(Generic[ScreenResultType], Widget):
         focusable_widgets = self.focus_chain
         if not focusable_widgets:
             # If there's nothing to focus... give up now.
+            self.set_focus(None)
             return
 
         try:
@@ -462,15 +518,34 @@ class Screen(Generic[ScreenResultType], Widget):
                 self.focused = widget
                 # Send focus event
                 if scroll_visible:
-                    self.screen.scroll_to_widget(widget)
+
+                    def scroll_to_center(widget: Widget) -> None:
+                        """Scroll to center (after a refresh)."""
+                        if widget.has_focus and not self.screen.can_view(widget):
+                            self.screen.scroll_to_center(widget, origin_visible=True)
+
+                    self.call_after_refresh(scroll_to_center, widget)
                 widget.post_message(events.Focus())
                 focused = widget
 
-                self._update_focus_styles(self.focused, widget)
                 self.log.debug(widget, "was focused")
 
         self._update_focus_styles(focused, blurred)
 
+    def _extend_compose(self, widgets: list[Widget]) -> None:
+        """Insert Textual's own internal widgets.
+
+        Args:
+            widgets: The list of widgets to be composed.
+
+        This method adds the tooltip, if required, and also adds the
+        container for `Toast`s.
+        """
+        if not self.app._disable_tooltips:
+            widgets.insert(0, Tooltip(id="textual-tooltip"))
+        if not self.app._disable_notifications:
+            widgets.insert(0, ToastRack(id="textual-toastrack"))
+
     async def _on_idle(self, event: events.Idle) -> None:
         # Check for any widgets marked as 'dirty' (needs a repaint)
         event.prevent_default()
@@ -522,17 +597,22 @@ class Screen(Generic[ScreenResultType], Widget):
         if self._callbacks:
             callbacks = self._callbacks[:]
             self._callbacks.clear()
-            for callback in callbacks:
-                await invoke(callback)
+            for callback, message_pump in callbacks:
+                reset_token = active_message_pump.set(message_pump)
+                try:
+                    await invoke(callback)
+                finally:
+                    active_message_pump.reset(reset_token)
 
-    def _invoke_later(self, callback: CallbackType) -> None:
+    def _invoke_later(self, callback: CallbackType, sender: MessagePump) -> None:
         """Enqueue a callback to be invoked after the screen is repainted.
 
         Args:
             callback: A callback.
+            sender: The sender (active message pump) of the callback.
         """
 
-        self._callbacks.append(callback)
+        self._callbacks.append((callback, sender))
         self.check_idle()
 
     def _push_result_callback(
@@ -577,6 +657,7 @@ class Screen(Generic[ScreenResultType], Widget):
                         virtual_size,
                         container_size,
                         _,
+                        _,
                     ) in layers:
                         if widget in exposed_widgets:
                             if widget._size_updated(
@@ -607,6 +688,7 @@ class Screen(Generic[ScreenResultType], Widget):
                     virtual_size,
                     container_size,
                     _,
+                    _,
                 ) in layers:
                     widget._size_updated(region.size, virtual_size, container_size)
                     if widget in send_resize:
@@ -658,9 +740,16 @@ class Screen(Generic[ScreenResultType], Widget):
     def _on_screen_resume(self) -> None:
         """Screen has resumed."""
         self.stack_updates += 1
+        self.app._refresh_notifications()
         size = self.app.size
         self._refresh_layout(size, full=True)
         self.refresh()
+        auto_focus = self.app.AUTO_FOCUS if self.AUTO_FOCUS is None else self.AUTO_FOCUS
+        if auto_focus and self.focused is None:
+            for widget in self.query(auto_focus):
+                if widget.focusable:
+                    self.set_focus(widget)
+                    break
 
     def _on_screen_suspend(self) -> None:
         """Screen has suspended."""
@@ -673,6 +762,43 @@ class Screen(Generic[ScreenResultType], Widget):
         for screen in self.app._background_screens:
             screen._screen_resized(event.size)
 
+    def _update_tooltip(self, widget: Widget) -> None:
+        """Update the content of the tooltip."""
+        try:
+            tooltip = self.get_child_by_type(Tooltip)
+        except NoMatches:
+            pass
+        else:
+            if tooltip.display and self._tooltip_widget is widget:
+                self._handle_tooltip_timer(widget)
+
+    def _handle_tooltip_timer(self, widget: Widget) -> None:
+        """Called by a timer from _handle_mouse_move to update the tooltip.
+
+        Args:
+            widget: The widget under the mouse.
+        """
+
+        try:
+            tooltip = self.get_child_by_type(Tooltip)
+        except NoMatches:
+            pass
+        else:
+            tooltip_content: RenderableType | None = None
+            for node in widget.ancestors_with_self:
+                if not isinstance(node, Widget):
+                    break
+                if node.tooltip is not None:
+                    tooltip_content = node.tooltip
+                    break
+
+            if tooltip_content is None:
+                tooltip.display = False
+            else:
+                tooltip.display = True
+                tooltip._absolute_offset = self.app.mouse_position
+                tooltip.update(tooltip_content)
+
     def _handle_mouse_move(self, event: events.MouseMove) -> None:
         try:
             if self.app.mouse_captured:
@@ -682,6 +808,14 @@ class Screen(Generic[ScreenResultType], Widget):
                 widget, region = self.get_widget_at(event.x, event.y)
         except errors.NoWidget:
             self.app._set_mouse_over(None)
+            if self._tooltip_timer is not None:
+                self._tooltip_timer.stop()
+            if not self.app._disable_tooltips:
+                try:
+                    self.get_child_by_type(Tooltip).display = False
+                except NoMatches:
+                    pass
+
         else:
             self.app._set_mouse_over(widget)
             mouse_event = events.MouseMove(
@@ -701,6 +835,27 @@ class Screen(Generic[ScreenResultType], Widget):
             mouse_event._set_forwarded()
             widget._forward_event(mouse_event)
 
+            if not self.app._disable_tooltips:
+                try:
+                    tooltip = self.get_child_by_type(Tooltip)
+                except NoMatches:
+                    pass
+                else:
+                    tooltip.styles.offset = event.screen_offset
+
+                    if self._tooltip_widget != widget or not tooltip.display:
+                        self._tooltip_widget = widget
+                        if self._tooltip_timer is not None:
+                            self._tooltip_timer.stop()
+
+                        self._tooltip_timer = self.set_timer(
+                            0.3,
+                            partial(self._handle_tooltip_timer, widget),
+                            name="tooltip-timer",
+                        )
+                    else:
+                        tooltip.display = False
+
     def _forward_event(self, event: events.Event) -> None:
         if event.is_forwarded:
             return
@@ -754,16 +909,23 @@ class Screen(Generic[ScreenResultType], Widget):
     def dismiss(self, result: ScreenResultType | Type[_NoResult] = _NoResult) -> None:
         """Dismiss the screen, optionally with a result.
 
+        If `result` is provided and a callback was set when the screen was [pushed][textual.app.App.push_screen], then
+        the callback will be invoked with `result`.
+
         Args:
             result: The optional result to be passed to the result callback.
 
-        Note:
-            If the screen was pushed with a callback, the callback will be
-            called with the given result and then a call to
-            [`App.pop_screen`][textual.app.App.pop_screen] is performed. If
-            no callback was provided calling this method is the same as
-            simply calling [`App.pop_screen`][textual.app.App.pop_screen].
+        Raises:
+            ScreenStackError: If trying to dismiss a screen that is not at the top of
+                the stack.
+
         """
+        if self is not self.app.screen:
+            from .app import ScreenStackError
+
+            raise ScreenStackError(
+                f"Can't dismiss screen {self} that's not at the top of the stack."
+            )
         if result is not self._NoResult and self._result_callbacks:
             self._result_callbacks[-1](cast(ScreenResultType, result))
         self.app.pop_screen()
@@ -778,6 +940,30 @@ class Screen(Generic[ScreenResultType], Widget):
         """
         self.dismiss(result)
 
+    def notify(
+        self,
+        message: str,
+        *,
+        title: str | None = None,
+        severity: SeverityLevel = "information",
+        timeout: float = Notification.timeout,
+    ) -> Notification:
+        """Create a notification.
+
+        Args:
+            message: The message for the notification.
+            title: The title for the notification.
+            severity: The severity of the notification.
+            timeout: The timeout for the notification.
+
+        Returns:
+            The new notification.
+
+        See [`App.notify`][textual.app.App.notify] for the full
+        documentation for this method.
+        """
+        return self.app.notify(message, title=title, severity=severity, timeout=timeout)
+
 
 @rich.repr.auto
 class ModalScreen(Screen[ScreenResultType]):
