@@ -29,11 +29,6 @@ class ScrollView(ScrollableContainer):
         """Always scrollable."""
         return True
 
-    @property
-    def is_transparent(self) -> bool:
-        """Not transparent, i.e. renders something."""
-        return False
-
     def watch_scroll_x(self, old_value: float, new_value: float) -> None:
         if self.show_horizontal_scrollbar and round(old_value) != round(new_value):
             self.horizontal_scrollbar.position = round(new_value)
@@ -122,6 +117,7 @@ class ScrollView(ScrollableContainer):
         duration: float | None = None,
         easing: EasingFunction | str | None = None,
         force: bool = False,
+        on_complete: CallbackType | None = None,
     ) -> None:
         """Scroll to a given (absolute) coordinate, optionally animating.
 
@@ -133,6 +129,7 @@ class ScrollView(ScrollableContainer):
             duration: Duration of animation, if `animate` is `True` and `speed` is `None`.
             easing: An easing method for the scrolling animation.
             force: Force scrolling even when prohibited by overflow styling.
+            on_complete: A callable to invoke when the animation is finished.
         """
 
         self._scroll_to(
@@ -143,4 +140,5 @@ class ScrollView(ScrollableContainer):
             duration=duration,
             easing=easing,
             force=force,
+            on_complete=on_complete,
         )
