@@ -7,11 +7,10 @@ See [line API](/guide/widgets#line-api) for how to use Strips.
 from __future__ import annotations
 
 from itertools import chain
-from typing import Iterable, Iterator, Sequence, cast
+from typing import Iterable, Iterator, Sequence
 
 import rich.repr
 from rich.cells import cell_len, set_cell_size
-from rich.color import ColorSystem
 from rich.console import Console, ConsoleOptions, RenderResult
 from rich.segment import Segment
 from rich.style import Style, StyleType
@@ -379,16 +378,22 @@ class Strip:
         """
 
         pos = 0
+        cell_length = self.cell_length
+        cuts = [cut for cut in cuts if cut <= cell_length]
         cache_key = tuple(cuts)
         cached = self._divide_cache.get(cache_key)
         if cached is not None:
             return cached
 
-        strips: list[Strip] = []
-        add_strip = strips.append
-        for segments, cut in zip(Segment.divide(self._segments, cuts), cuts):
-            add_strip(Strip(segments, cut - pos))
-            pos = cut
+        strips: list[Strip]
+        if cuts == [cell_length]:
+            strips = [self]
+        else:
+            strips = []
+            add_strip = strips.append
+            for segments, cut in zip(Segment.divide(self._segments, cuts), cuts):
+                add_strip(Strip(segments, cut - pos))
+                pos = cut
 
         self._divide_cache[cache_key] = strips
         return strips
