@@ -29,11 +29,15 @@ if typing.TYPE_CHECKING:
     from ._progress_bar import ProgressBar
     from ._radio_button import RadioButton
     from ._radio_set import RadioSet
+    from ._select import Select
+    from ._selection_list import SelectionList
+    from ._sparkline import Sparkline
     from ._static import Static
     from ._switch import Switch
     from ._tabbed_content import TabbedContent, TabPane
     from ._tabs import Tab, Tabs
     from ._text_log import TextLog
+    from ._tooltip import Tooltip
     from ._tree import Tree
     from ._welcome import Welcome
 
@@ -59,6 +63,9 @@ __all__ = [
     "ProgressBar",
     "RadioButton",
     "RadioSet",
+    "Select",
+    "SelectionList",
+    "Sparkline",
     "Static",
     "Switch",
     "Tab",
@@ -66,6 +73,7 @@ __all__ = [
     "TabPane",
     "Tabs",
     "TextLog",
+    "Tooltip",
     "Tree",
     "Welcome",
 ]
