@@ -38,6 +38,7 @@ CellCacheKey: TypeAlias = (
 LineCacheKey: TypeAlias = "tuple[int, int, int, int, Coordinate, Coordinate, Style, CursorType, bool, int, PseudoClasses]"
 RowCacheKey: TypeAlias = "tuple[RowKey, int, Style, Coordinate, Coordinate, CursorType, bool, bool, int, PseudoClasses]"
 CursorType = Literal["cell", "row", "column", "none"]
+"""The legal types of cursors for [`DataTable.cursor_type`][textual.widgets.DataTable.cursor_type]."""
 CellType = TypeVar("CellType")
 
 CELL_X_PADDING = 2
@@ -96,7 +97,7 @@ class StringKey:
             else:
                 return hash(self) == hash(other)
         else:
-            raise NotImplemented
+            return NotImplemented
 
     def __lt__(self, other):
         if isinstance(other, str):
@@ -104,7 +105,7 @@ class StringKey:
         elif isinstance(other, StringKey):
             return self.value < other.value
         else:
-            raise NotImplemented
+            return NotImplemented
 
     def __rich_repr__(self):
         yield "value", self.value
@@ -304,14 +305,21 @@ class DataTable(ScrollView, Generic[CellType], can_focus=True):
     zebra_stripes = Reactive(False)
     header_height = Reactive(1)
     show_cursor = Reactive(True)
-    cursor_type = Reactive("cell")
+    cursor_type: Reactive[CursorType] = Reactive[CursorType]("cell")
+    """The type of the cursor of the `DataTable`."""
 
     cursor_coordinate: Reactive[Coordinate] = Reactive(
         Coordinate(0, 0), repaint=False, always_update=True
     )
+    """Current cursor [`Coordinate`][textual.coordinate.Coordinate].
+
+    This can be set programmatically or changed via the method
+    [`move_cursor`][textual.widgets.DataTable.move_cursor].
+    """
     hover_coordinate: Reactive[Coordinate] = Reactive(
         Coordinate(0, 0), repaint=False, always_update=True
     )
+    """The coordinate of the `DataTable` that is being hovered."""
 
     class CellHighlighted(Message, bubble=True):
         """Posted when the cursor moves to highlight a new cell.
@@ -571,6 +579,8 @@ class DataTable(ScrollView, Generic[CellType], can_focus=True):
         zebra_stripes: bool = False,
         header_height: int = 1,
         show_cursor: bool = True,
+        cursor_foreground_priority: Literal["renderable", "css"] = "css",
+        cursor_background_priority: Literal["renderable", "css"] = "renderable",
         name: str | None = None,
         id: str | None = None,
         classes: str | None = None,
@@ -652,6 +662,12 @@ class DataTable(ScrollView, Generic[CellType], can_focus=True):
         """Apply zebra effect on row backgrounds (light, dark, light, dark, ...)."""
         self.show_cursor = show_cursor
         """Show/hide both the keyboard and hover cursor."""
+        self.cursor_foreground_priority = cursor_foreground_priority
+        """Should we prioritize the cursor component class CSS foreground or the renderable foreground
+         in the event where a cell contains a renderable with a foreground color."""
+        self.cursor_background_priority = cursor_background_priority
+        """Should we prioritize the cursor component class CSS background or the renderable background
+         in the event where a cell contains a renderable with a background color."""
 
     @property
     def hover_row(self) -> int:
@@ -756,7 +772,7 @@ class DataTable(ScrollView, Generic[CellType], can_focus=True):
         row_key, column_key = self.coordinate_to_cell_key(coordinate)
         self.update_cell(row_key, column_key, value, update_width=update_width)
 
-    def get_cell(self, row_key: RowKey, column_key: ColumnKey) -> CellType:
+    def get_cell(self, row_key: RowKey | str, column_key: ColumnKey | str) -> CellType:
         """Given a row key and column key, return the value of the corresponding cell.
 
         Args:
@@ -789,6 +805,32 @@ class DataTable(ScrollView, Generic[CellType], can_focus=True):
         row_key, column_key = self.coordinate_to_cell_key(coordinate)
         return self.get_cell(row_key, column_key)
 
+    def get_cell_coordinate(
+        self, row_key: RowKey | str, column_key: Column | str
+    ) -> Coordinate:
+        """Given a row key and column key, return the corresponding cell coordinate.
+
+        Args:
+            row_key: The row key of the cell.
+            column_key: The column key of the cell.
+
+        Returns:
+            The current coordinate of the cell identified by the row and column keys.
+
+        Raises:
+            CellDoesNotExist: If the specified cell does not exist.
+        """
+        if (
+            row_key not in self._row_locations
+            or column_key not in self._column_locations
+        ):
+            raise CellDoesNotExist(
+                f"No cell exists for row_key={row_key!r}, column_key={column_key!r}."
+            )
+        row_index = self._row_locations.get(row_key)
+        column_index = self._column_locations.get(column_key)
+        return Coordinate(row_index, column_index)
+
     def get_row(self, row_key: RowKey | str) -> list[CellType]:
         """Get the values from the row identified by the given row key.
 
@@ -828,6 +870,22 @@ class DataTable(ScrollView, Generic[CellType], can_focus=True):
         row_key = self._row_locations.get_key(row_index)
         return self.get_row(row_key)
 
+    def get_row_index(self, row_key: RowKey | str) -> int:
+        """Return the current index for the row identified by row_key.
+
+        Args:
+            row_key: The row key to find the current index of.
+
+        Returns:
+            The current index of the specified row key.
+
+        Raises:
+            RowDoesNotExist: If the row key does not exist.
+        """
+        if row_key not in self._row_locations:
+            raise RowDoesNotExist(f"No row exists for row_key={row_key!r}")
+        return self._row_locations.get(row_key)
+
     def get_column(self, column_key: ColumnKey | str) -> Iterable[CellType]:
         """Get the values from the column identified by the given column key.
 
@@ -866,6 +924,22 @@ class DataTable(ScrollView, Generic[CellType], can_focus=True):
         column_key = self._column_locations.get_key(column_index)
         yield from self.get_column(column_key)
 
+    def get_column_index(self, column_key: ColumnKey | str) -> int:
+        """Return the current index for the column identified by column_key.
+
+        Args:
+            column_key: The column key to find the current index of.
+
+        Returns:
+            The current index of the specified column key.
+
+        Raises:
+            ColumnDoesNotExist: If the column key does not exist.
+        """
+        if column_key not in self._column_locations:
+            raise ColumnDoesNotExist(f"No column exists for column_key={column_key!r}")
+        return self._column_locations.get(column_key)
+
     def _clear_caches(self) -> None:
         self._row_render_cache.clear()
         self._cell_render_cache.clear()
@@ -950,6 +1024,41 @@ class DataTable(ScrollView, Generic[CellType], can_focus=True):
             elif self.cursor_type == "column":
                 self.refresh_column(old_coordinate.column)
                 self._highlight_column(new_coordinate.column)
+            # If the coordinate was changed via `move_cursor`, give priority to its
+            # scrolling because it may be animated.
+            self.call_next(self._scroll_cursor_into_view)
+
+    def move_cursor(
+        self,
+        *,
+        row: int | None = None,
+        column: int | None = None,
+        animate: bool = False,
+    ) -> None:
+        """Move the cursor to the given position.
+
+        Example:
+            ```py
+            datatable = app.query_one(DataTable)
+            datatable.move_cursor(row=4, column=6)
+            # datatable.cursor_coordinate == Coordinate(4, 6)
+            datatable.move_cursor(row=3)
+            # datatable.cursor_coordinate == Coordinate(3, 6)
+            ```
+
+        Args:
+            row: The new row to move the cursor to.
+            column: The new column to move the cursor to.
+            animate: Whether to animate the change of coordinates.
+        """
+        cursor_row, cursor_column = self.cursor_coordinate
+        if row is not None:
+            cursor_row = row
+        if column is not None:
+            cursor_column = column
+        destination = Coordinate(cursor_row, cursor_column)
+        self.cursor_coordinate = destination
+        self._scroll_cursor_into_view(animate=animate)
 
     def _highlight_coordinate(self, coordinate: Coordinate) -> None:
         """Apply highlighting to the cell at the coordinate, and post event."""
@@ -1074,6 +1183,8 @@ class DataTable(ScrollView, Generic[CellType], can_focus=True):
             else:
                 column.content_width = max(new_content_width, label_width)
 
+        self._require_update_dimensions = True
+
     def _update_dimensions(self, new_rows: Iterable[RowKey]) -> None:
         """Called to recalculate the virtual (scrollable) size."""
         console = self.app.console
@@ -1142,7 +1253,10 @@ class DataTable(ScrollView, Generic[CellType], can_focus=True):
         rows = self.rows
         row_key = self._row_locations.get_key(row_index)
         row = rows[row_key]
-        row_width = sum(column.render_width for column in self.columns.values())
+        row_width = (
+            sum(column.render_width for column in self.columns.values())
+            + self._row_label_column_width
+        )
         y = sum(ordered_row.height for ordered_row in self.ordered_rows[:row_index])
         if self.show_header:
             y += self.header_height
@@ -1196,7 +1310,12 @@ class DataTable(ScrollView, Generic[CellType], can_focus=True):
         return self
 
     def add_column(
-        self, label: TextType, *, width: int | None = None, key: str | None = None
+        self,
+        label: TextType,
+        *,
+        width: int | None = None,
+        key: str | None = None,
+        default: CellType | None = None,
     ) -> ColumnKey:
         """Add a column to the table.
 
@@ -1205,6 +1324,7 @@ class DataTable(ScrollView, Generic[CellType], can_focus=True):
             width: Width of the column in cells or None to fit content.
             key: A key which uniquely identifies this column.
                 If None, it will be generated for you.
+            default: The  value to insert into pre-existing rows.
 
         Returns:
             Uniquely identifies this column. Can be used to retrieve this column
@@ -1235,6 +1355,12 @@ class DataTable(ScrollView, Generic[CellType], can_focus=True):
 
         self.columns[column_key] = column
         self._column_locations[column_key] = column_index
+
+        # Update pre-existing rows to account for the new column.
+        for row_key in self.rows.keys():
+            self._data[row_key][column_key] = default
+            self._updated_cells.add(CellKey(row_key, column_key))
+
         self._require_update_dimensions = True
         self.check_idle()
 
@@ -1364,6 +1490,42 @@ class DataTable(ScrollView, Generic[CellType], can_focus=True):
         self._update_count += 1
         self.refresh(layout=True)
 
+    def remove_column(self, column_key: ColumnKey | str) -> None:
+        """Remove a column (identified by a key) from the DataTable.
+
+        Args:
+            column_key: The key identifying the column to remove.
+
+        Raises:
+            ColumnDoesNotExist: If the column key does not exist.
+        """
+        if column_key not in self._column_locations:
+            raise ColumnDoesNotExist(f"Column key {column_key!r} is not valid.")
+
+        self._require_update_dimensions = True
+        self.check_idle()
+
+        index_to_delete = self._column_locations.get(column_key)
+        new_column_locations = TwoWayDict({})
+        for column_location_key in self._column_locations:
+            column_index = self._column_locations.get(column_location_key)
+            if column_index > index_to_delete:
+                new_column_locations[column_location_key] = column_index - 1
+            elif column_index < index_to_delete:
+                new_column_locations[column_location_key] = column_index
+
+        self._column_locations = new_column_locations
+
+        del self.columns[column_key]
+        for row in self._data:
+            del self._data[row][column_key]
+
+        self.cursor_coordinate = self.cursor_coordinate
+        self.hover_coordinate = self.hover_coordinate
+
+        self._update_count += 1
+        self.refresh(layout=True)
+
     async def _on_idle(self, _: events.Idle) -> None:
         """Runs when the message pump is empty.
 
@@ -1371,6 +1533,13 @@ class DataTable(ScrollView, Generic[CellType], can_focus=True):
         whole DataTable and re-computing column widths after some cells
         have been updated. This is more efficient in the case of high
         frequency updates, ensuring we only do expensive computations once."""
+        if self._updated_cells:
+            # Cell contents have already been updated at this point.
+            # Now we only need to worry about measuring column widths.
+            updated_columns = self._updated_cells.copy()
+            self._updated_cells.clear()
+            self._update_column_widths(updated_columns)
+
         if self._require_update_dimensions:
             # Add the new rows *before* updating the column widths, since
             # cells in a new row may influence the final width of a column
@@ -1379,13 +1548,6 @@ class DataTable(ScrollView, Generic[CellType], can_focus=True):
             self._new_rows.clear()
             self._update_dimensions(new_rows)
 
-        if self._updated_cells:
-            # Cell contents have already been updated at this point.
-            # Now we only need to worry about measuring column widths.
-            updated_columns = self._updated_cells.copy()
-            self._updated_cells.clear()
-            self._update_column_widths(updated_columns)
-
     def refresh_coordinate(self, coordinate: Coordinate) -> Self:
         """Refresh the cell at a coordinate.
 
@@ -1608,38 +1770,42 @@ class DataTable(ScrollView, Generic[CellType], can_focus=True):
             else:
                 cell = row_cells[column_index]
 
-            get_component = self.get_component_styles
+            get_component = self.get_component_rich_style
             show_cursor = self.show_cursor
-
             component_style = Style()
 
             if hover and show_cursor and self._show_hover_cursor:
-                component_style += get_component("datatable--hover").rich_style
+                component_style += get_component("datatable--hover")
                 if is_header_cell or is_row_label_cell:
                     # Apply subtle variation in style for the header/label (blue background by
                     # default) rows and columns affected by the cursor, to ensure we can
                     # still differentiate between the labels and the data.
-                    component_style += get_component(
-                        "datatable--header-hover"
-                    ).rich_style
+                    component_style += get_component("datatable--header-hover")
 
             if cursor and show_cursor:
-                cursor_style = get_component("datatable--cursor").rich_style
+                cursor_style = get_component("datatable--cursor")
                 component_style += cursor_style
                 if is_header_cell or is_row_label_cell:
-                    component_style += get_component(
-                        "datatable--header-cursor"
-                    ).rich_style
+                    component_style += get_component("datatable--header-cursor")
                 elif is_fixed_style_cell:
-                    component_style += get_component(
-                        "datatable--fixed-cursor"
-                    ).rich_style
+                    component_style += get_component("datatable--fixed-cursor")
+
+            post_foreground = (
+                Style.from_color(color=component_style.color)
+                if self.cursor_foreground_priority == "css"
+                else Style.null()
+            )
+            post_background = (
+                Style.from_color(bgcolor=component_style.bgcolor)
+                if self.cursor_background_priority == "css"
+                else Style.null()
+            )
 
             lines = self.app.console.render_lines(
                 Styled(
                     Padding(cell, (0, 1)),
-                    pre_style=base_style,
-                    post_style=component_style,
+                    pre_style=base_style + component_style,
+                    post_style=post_foreground + post_background,
                 ),
                 self.app.console.options.update_dimensions(width, height),
             )
@@ -2051,7 +2217,6 @@ class DataTable(ScrollView, Generic[CellType], can_focus=True):
             self.cursor_coordinate = Coordinate(
                 row_index + rows_to_scroll - 1, column_index
             )
-            self._scroll_cursor_into_view()
         else:
             super().action_page_down()
 
@@ -2075,7 +2240,6 @@ class DataTable(ScrollView, Generic[CellType], can_focus=True):
             self.cursor_coordinate = Coordinate(
                 row_index - rows_to_scroll + 1, column_index
             )
-            self._scroll_cursor_into_view()
         else:
             super().action_page_up()
 
@@ -2086,7 +2250,6 @@ class DataTable(ScrollView, Generic[CellType], can_focus=True):
         if self.show_cursor and (cursor_type == "cell" or cursor_type == "row"):
             row_index, column_index = self.cursor_coordinate
             self.cursor_coordinate = Coordinate(0, column_index)
-            self._scroll_cursor_into_view()
         else:
             super().action_scroll_home()
 
@@ -2097,7 +2260,6 @@ class DataTable(ScrollView, Generic[CellType], can_focus=True):
         if self.show_cursor and (cursor_type == "cell" or cursor_type == "row"):
             row_index, column_index = self.cursor_coordinate
             self.cursor_coordinate = Coordinate(self.row_count - 1, column_index)
-            self._scroll_cursor_into_view()
         else:
             super().action_scroll_end()
 
@@ -2106,7 +2268,6 @@ class DataTable(ScrollView, Generic[CellType], can_focus=True):
         cursor_type = self.cursor_type
         if self.show_cursor and (cursor_type == "cell" or cursor_type == "row"):
             self.cursor_coordinate = self.cursor_coordinate.up()
-            self._scroll_cursor_into_view()
         else:
             # If the cursor doesn't move up (e.g. column cursor can't go up),
             # then ensure that we instead scroll the DataTable.
@@ -2117,7 +2278,6 @@ class DataTable(ScrollView, Generic[CellType], can_focus=True):
         cursor_type = self.cursor_type
         if self.show_cursor and (cursor_type == "cell" or cursor_type == "row"):
             self.cursor_coordinate = self.cursor_coordinate.down()
-            self._scroll_cursor_into_view()
         else:
             super().action_scroll_down()
 
