@@ -4,6 +4,7 @@ from pathlib import Path, PurePath
 from typing import Callable, Iterable
 
 from markdown_it import MarkdownIt
+from markdown_it.token import Token
 from rich import box
 from rich.style import Style
 from rich.syntax import Syntax
@@ -12,11 +13,11 @@ from rich.text import Text
 from typing_extensions import TypeAlias
 
 from ..app import ComposeResult
-from ..containers import Horizontal, VerticalScroll
+from ..containers import Horizontal, Vertical, VerticalScroll
 from ..events import Mount
 from ..message import Message
 from ..reactive import reactive, var
-from ..widget import Widget
+from ..widget import AwaitMount, Widget
 from ..widgets import Static, Tree
 
 TableOfContentsType: TypeAlias = "list[tuple[int, str, str | None]]"
@@ -88,7 +89,9 @@ class MarkdownBlock(Static):
     }
     """
 
-    def __init__(self, *args, **kwargs) -> None:
+    def __init__(self, markdown: Markdown, *args, **kwargs) -> None:
+        self._markdown: Markdown = markdown
+        """A reference to the Markdown document that contains this block."""
         self._text = Text()
         self._blocks: list[MarkdownBlock] = []
         super().__init__(*args, **kwargs)
@@ -103,7 +106,7 @@ class MarkdownBlock(Static):
 
     async def action_link(self, href: str) -> None:
         """Called on link click."""
-        self.post_message(Markdown.LinkClicked(href))
+        self.post_message(Markdown.LinkClicked(self._markdown, href))
 
 
 class MarkdownHeader(MarkdownBlock):
@@ -267,7 +270,7 @@ class MarkdownBulletList(MarkdownList):
         width: 1fr;
     }
 
-    MarkdownBulletList VerticalScroll {
+    MarkdownBulletList Vertical {
         height: auto;
         width: 1fr;
     }
@@ -278,7 +281,7 @@ class MarkdownBulletList(MarkdownList):
             if isinstance(block, MarkdownListItem):
                 bullet = MarkdownBullet()
                 bullet.symbol = block.bullet
-                yield Horizontal(bullet, VerticalScroll(*block._blocks))
+                yield Horizontal(bullet, Vertical(*block._blocks))
         self._blocks.clear()
 
 
@@ -296,7 +299,7 @@ class MarkdownOrderedList(MarkdownList):
         width: 1fr;
     }
 
-    MarkdownOrderedList VerticalScroll {
+    MarkdownOrderedList Vertical {
         height: auto;
         width: 1fr;
     }
@@ -319,7 +322,7 @@ class MarkdownOrderedList(MarkdownList):
             if isinstance(block, MarkdownListItem):
                 bullet = MarkdownBullet()
                 bullet.symbol = f"{number}{suffix}".rjust(symbol_size + 1)
-                yield Horizontal(bullet, VerticalScroll(*block._blocks))
+                yield Horizontal(bullet, Vertical(*block._blocks))
 
         self._blocks.clear()
 
@@ -447,15 +450,15 @@ class MarkdownListItem(MarkdownBlock):
         height: auto;
     }
 
-    MarkdownListItem > VerticalScroll {
+    MarkdownListItem > Vertical {
         width: 1fr;
         height: auto;
     }
     """
 
-    def __init__(self, bullet: str) -> None:
+    def __init__(self, markdown: Markdown, bullet: str) -> None:
         self.bullet = bullet
-        super().__init__()
+        super().__init__(markdown)
 
 
 class MarkdownOrderedListItem(MarkdownListItem):
@@ -484,10 +487,10 @@ class MarkdownFence(MarkdownBlock):
     }
     """
 
-    def __init__(self, code: str, lexer: str) -> None:
+    def __init__(self, markdown: Markdown, code: str, lexer: str) -> None:
         self.code = code
         self.lexer = lexer
-        super().__init__()
+        super().__init__(markdown)
 
     def compose(self) -> ComposeResult:
         yield Static(
@@ -565,27 +568,62 @@ class Markdown(Widget):
     class TableOfContentsUpdated(Message, bubble=True):
         """The table of contents was updated."""
 
-        def __init__(self, table_of_contents: TableOfContentsType) -> None:
+        def __init__(
+            self, markdown: Markdown, table_of_contents: TableOfContentsType
+        ) -> None:
             super().__init__()
+            self.markdown: Markdown = markdown
+            """The `Markdown` widget associated with the table of contents."""
             self.table_of_contents: TableOfContentsType = table_of_contents
             """Table of contents."""
 
+        @property
+        def control(self) -> Markdown:
+            """The `Markdown` widget associated with the table of contents.
+
+            This is an alias for [`TableOfContentsUpdated.markdown`][textual.widgets.Markdown.TableOfContentsSelected.markdown]
+            and is used by the [`on`][textual.on] decorator.
+            """
+            return self.markdown
+
     class TableOfContentsSelected(Message, bubble=True):
         """An item in the TOC was selected."""
 
-        def __init__(self, block_id: str) -> None:
+        def __init__(self, markdown: Markdown, block_id: str) -> None:
             super().__init__()
-            self.block_id = block_id
+            self.markdown: Markdown = markdown
+            """The `Markdown` widget where the selected item is."""
+            self.block_id: str = block_id
             """ID of the block that was selected."""
 
+        @property
+        def control(self) -> Markdown:
+            """The `Markdown` widget where the selected item is.
+
+            This is an alias for [`TableOfContentsSelected.markdown`][textual.widgets.Markdown.TableOfContentsSelected.markdown]
+            and is used by the [`on`][textual.on] decorator.
+            """
+            return self.markdown
+
     class LinkClicked(Message, bubble=True):
         """A link in the document was clicked."""
 
-        def __init__(self, href: str) -> None:
+        def __init__(self, markdown: Markdown, href: str) -> None:
             super().__init__()
+            self.markdown: Markdown = markdown
+            """The `Markdown` widget containing the link clicked."""
             self.href: str = href
             """The link that was selected."""
 
+        @property
+        def control(self) -> Markdown:
+            """The `Markdown` widget containing the link clicked.
+
+            This is an alias for [`LinkClicked.markdown`][textual.widgets.Markdown.LinkClicked.markdown]
+            and is used by the [`on`][textual.on] decorator.
+            """
+            return self.markdown
+
     def _on_mount(self, _: Mount) -> None:
         if self._markdown is not None:
             self.update(self._markdown)
@@ -597,21 +635,35 @@ class Markdown(Widget):
             path: Path to the document.
 
         Returns:
-            True    on success, or False if the document could not be read.
+            True on success, or False if the document could not be read.
         """
         try:
             markdown = path.read_text(encoding="utf-8")
         except Exception:
             return False
 
-        self.update(markdown)
+        await self.update(markdown)
         return True
 
-    def update(self, markdown: str) -> None:
+    def unhandled_token(self, token: Token) -> MarkdownBlock | None:
+        """Process an unhandled token.
+
+        Args:
+            token: The token to handle.
+
+        Returns:
+            Either a widget to be added to the output, or `None`.
+        """
+        return None
+
+    def update(self, markdown: str) -> AwaitMount:
         """Update the document with new Markdown.
 
         Args:
             markdown: A string containing Markdown.
+
+        Returns:
+            An optionally awaitable object. Await this to ensure that all children have been mounted.
         """
         output: list[MarkdownBlock] = []
         stack: list[MarkdownBlock] = []
@@ -621,28 +673,26 @@ class Markdown(Widget):
             else self._parser_factory()
         )
 
-        content = Text()
         block_id: int = 0
-
         table_of_contents: TableOfContentsType = []
 
         for token in parser.parse(markdown):
             if token.type == "heading_open":
                 block_id += 1
-                stack.append(HEADINGS[token.tag](id=f"block{block_id}"))
+                stack.append(HEADINGS[token.tag](self, id=f"block{block_id}"))
             elif token.type == "hr":
-                output.append(MarkdownHorizontalRule())
+                output.append(MarkdownHorizontalRule(self))
             elif token.type == "paragraph_open":
-                stack.append(MarkdownParagraph())
+                stack.append(MarkdownParagraph(self))
             elif token.type == "blockquote_open":
-                stack.append(MarkdownBlockQuote())
+                stack.append(MarkdownBlockQuote(self))
             elif token.type == "bullet_list_open":
-                stack.append(MarkdownBulletList())
+                stack.append(MarkdownBulletList(self))
             elif token.type == "ordered_list_open":
-                stack.append(MarkdownOrderedList())
+                stack.append(MarkdownOrderedList(self))
             elif token.type == "list_item_open":
                 if token.info:
-                    stack.append(MarkdownOrderedListItem(token.info))
+                    stack.append(MarkdownOrderedListItem(self, token.info))
                 else:
                     item_count = sum(
                         1
@@ -651,22 +701,23 @@ class Markdown(Widget):
                     )
                     stack.append(
                         MarkdownUnorderedListItem(
-                            self.BULLETS[item_count % len(self.BULLETS)]
+                            self,
+                            self.BULLETS[item_count % len(self.BULLETS)],
                         )
                     )
 
             elif token.type == "table_open":
-                stack.append(MarkdownTable())
+                stack.append(MarkdownTable(self))
             elif token.type == "tbody_open":
-                stack.append(MarkdownTBody())
+                stack.append(MarkdownTBody(self))
             elif token.type == "thead_open":
-                stack.append(MarkdownTHead())
+                stack.append(MarkdownTHead(self))
             elif token.type == "tr_open":
-                stack.append(MarkdownTR())
+                stack.append(MarkdownTR(self))
             elif token.type == "th_open":
-                stack.append(MarkdownTH())
+                stack.append(MarkdownTH(self))
             elif token.type == "td_open":
-                stack.append(MarkdownTD())
+                stack.append(MarkdownTD(self))
             elif token.type.endswith("_close"):
                 block = stack.pop()
                 if token.type == "heading_close":
@@ -687,7 +738,7 @@ class Markdown(Widget):
                         if child.type == "hardbreak":
                             content.append("\n")
                         if child.type == "softbreak":
-                            content.append(" ")
+                            content.append(" ", style_stack[-1])
                         elif child.type == "code_inline":
                             content.append(
                                 child.content,
@@ -739,18 +790,23 @@ class Markdown(Widget):
                             style_stack.pop()
 
                 stack[-1].set_content(content)
-            elif token.type == "fence":
-                output.append(
+            elif token.type in ("fence", "code_block"):
+                (stack[-1]._blocks if stack else output).append(
                     MarkdownFence(
+                        self,
                         token.content.rstrip(),
                         token.info,
                     )
                 )
+            else:
+                external = self.unhandled_token(token)
+                if external is not None:
+                    (stack[-1]._blocks if stack else output).append(external)
 
-        self.post_message(Markdown.TableOfContentsUpdated(table_of_contents))
+        self.post_message(Markdown.TableOfContentsUpdated(self, table_of_contents))
         with self.app.batch_update():
             self.query("MarkdownBlock").remove()
-            self.mount_all(output)
+            return self.mount_all(output)
 
 
 class MarkdownTableOfContents(Widget, can_focus_children=True):
@@ -768,6 +824,27 @@ class MarkdownTableOfContents(Widget, can_focus_children=True):
 
     table_of_contents = reactive["TableOfContentsType | None"](None, init=False)
 
+    def __init__(
+        self,
+        markdown: Markdown,
+        name: str | None = None,
+        id: str | None = None,
+        classes: str | None = None,
+        disabled: bool = False,
+    ) -> None:
+        """Initialize a table of contents.
+
+        Args:
+            markdown: The Markdown document associated with this table of contents.
+            name: The name of the widget.
+            id: The ID of the widget in the DOM.
+            classes: The CSS classes for the widget.
+            disabled: Whether the widget is disabled or not.
+        """
+        self.markdown = markdown
+        """The Markdown document associated with this table of contents."""
+        super().__init__(name=name, id=id, classes=classes, disabled=disabled)
+
     def compose(self) -> ComposeResult:
         tree: Tree = Tree("TOC")
         tree.show_root = False
@@ -804,8 +881,9 @@ class MarkdownTableOfContents(Widget, can_focus_children=True):
         node_data = message.node.data
         if node_data is not None:
             await self._post_message(
-                Markdown.TableOfContentsSelected(node_data["block_id"])
+                Markdown.TableOfContentsSelected(self.markdown, node_data["block_id"])
             )
+        message.stop()
 
 
 class MarkdownViewer(VerticalScroll, can_focus=True, can_focus_children=True):
@@ -896,8 +974,9 @@ class MarkdownViewer(VerticalScroll, can_focus=True, can_focus_children=True):
         self.set_class(show_table_of_contents, "-show-table-of-contents")
 
     def compose(self) -> ComposeResult:
-        yield MarkdownTableOfContents()
-        yield Markdown(parser_factory=self._parser_factory)
+        markdown = Markdown(parser_factory=self._parser_factory)
+        yield MarkdownTableOfContents(markdown)
+        yield markdown
 
     def _on_markdown_table_of_contents_updated(
         self, message: Markdown.TableOfContentsUpdated
