@@ -25,23 +25,27 @@ class RadioSet(Container, can_focus=True, can_focus_children=False):
 
     DEFAULT_CSS = """
     RadioSet {
-        border: round #666;
+        border: tall transparent;
+        background: $boost;
+        padding: 0 1 0 0;
         height: auto;
         width: auto;
     }
 
     RadioSet:focus {
-        border: round $accent;
-    }
-
-    App.-light-mode RadioSet {
-        border: round #CCC;
+        border: tall $accent;
     }
 
     /* The following rules/styles mimic similar ToggleButton:focus rules in
      * ToggleButton. If those styles ever get updated, these should be too.
      */
 
+    RadioSet > * {
+        background: transparent;
+        border: none;
+        padding: 0 1;
+    }
+
     RadioSet:focus > RadioButton.-selected > .toggle--label {
         text-style: underline;
     }
@@ -78,6 +82,9 @@ class RadioSet(Container, can_focus=True, can_focus_children=False):
         This message can be handled using an `on_radio_set_changed` method.
         """
 
+        ALLOW_SELECTOR_MATCH = {"pressed"}
+        """Additional message attributes that can be used with the [`on` decorator][textual.on]."""
+
         def __init__(self, radio_set: RadioSet, pressed: RadioButton) -> None:
             """Initialise the message.
 
@@ -92,6 +99,15 @@ class RadioSet(Container, can_focus=True, can_focus_children=False):
             self.index = radio_set.pressed_index
             """The index of the [`RadioButton`][textual.widgets.RadioButton] that was pressed to make the change."""
 
+        @property
+        def control(self) -> RadioSet:
+            """A reference to the [`RadioSet`][textual.widgets.RadioSet] that was changed.
+
+            This is an alias for [`Changed.radio_set`][textual.widgets.RadioSet.Changed.radio_set]
+            and is used by the [`on`][textual.on] decorator.
+            """
+            return self.radio_set
+
         def __rich_repr__(self) -> rich.repr.Result:
             yield "radio_set", self.radio_set
             yield "pressed", self.pressed
