@@ -98,7 +98,7 @@ class TreeNode(Generic[TreeDataType]):
             id: The ID of the node.
             label: The label for the node.
             data: Optional data to associate with the node.
-            expand: Should the node be attached in an expanded state?
+            expanded: Should the node be attached in an expanded state?
             allow_expand: Should the node allow being expanded by the user?
         """
         self._tree = tree
@@ -184,6 +184,11 @@ class TreeNode(Generic[TreeDataType]):
             self._parent._children and self._parent._children[-1] == self,
         )
 
+    @property
+    def is_root(self) -> bool:
+        """Is this node the root of the tree?"""
+        return self == self._tree.root
+
     @property
     def allow_expand(self) -> bool:
         """Is this node allowed to expand?"""
@@ -202,6 +207,7 @@ class TreeNode(Generic[TreeDataType]):
         """
         self._expanded = True
         self._updates += 1
+        self._tree.post_message(Tree.NodeExpanded(self))
         if expand_all:
             for child in self.children:
                 child._expand(expand_all)
@@ -234,6 +240,7 @@ class TreeNode(Generic[TreeDataType]):
         """
         self._expanded = False
         self._updates += 1
+        self._tree.post_message(Tree.NodeCollapsed(self))
         if collapse_all:
             for child in self.children:
                 child._collapse(collapse_all)
@@ -300,6 +307,7 @@ class TreeNode(Generic[TreeDataType]):
         self._updates += 1
         text_label = self._tree.process_label(label)
         self._label = text_label
+        self._tree.call_later(self._tree._refresh_node, self)
 
     def add(
         self,
@@ -344,6 +352,47 @@ class TreeNode(Generic[TreeDataType]):
         node = self.add(label, data, expand=False, allow_expand=False)
         return node
 
+    class RemoveRootError(Exception):
+        """Exception raised when trying to remove a tree's root node."""
+
+    def _remove_children(self) -> None:
+        """Remove child nodes of this node.
+
+        Note:
+            This is the internal support method for `remove_children`. Call
+            `remove_children` to ensure the tree gets refreshed.
+        """
+        for child in reversed(self._children):
+            child._remove()
+
+    def _remove(self) -> None:
+        """Remove the current node and all its children.
+
+        Note:
+            This is the internal support method for `remove`. Call `remove`
+            to ensure the tree gets refreshed.
+        """
+        self._remove_children()
+        assert self._parent is not None
+        del self._parent._children[self._parent._children.index(self)]
+        del self._tree._tree_nodes[self.id]
+
+    def remove(self) -> None:
+        """Remove this node from the tree.
+
+        Raises:
+            TreeNode.RemoveRootError: If there is an attempt to remove the root.
+        """
+        if self.is_root:
+            raise self.RemoveRootError("Attempt to remove the root node of a Tree.")
+        self._remove()
+        self._tree._invalidate()
+
+    def remove_children(self) -> None:
+        """Remove any child nodes of this node."""
+        self._remove_children()
+        self._tree._invalidate()
+
 
 class Tree(Generic[TreeDataType], ScrollView, can_focus=True):
     """A widget for displaying and navigating data in a tree."""
@@ -429,7 +478,7 @@ class Tree(Generic[TreeDataType], ScrollView, can_focus=True):
     """Show the root of the tree."""
     hover_line = var(-1)
     """The line number under the mouse pointer, or -1 if not under the mouse pointer."""
-    cursor_line = var(-1)
+    cursor_line = var(-1, always_update=True)
     """The line with the cursor, or -1 if no cursor."""
     show_guides = reactive(True)
     """Enable display of tree guide lines."""
@@ -464,57 +513,69 @@ class Tree(Generic[TreeDataType], ScrollView, can_focus=True):
 
         Can be handled using `on_tree_node_collapsed` in a subclass of `Tree` or in a
         parent node in the DOM.
-
-        Attributes:
-            node: The node that was collapsed.
         """
 
         def __init__(self, node: TreeNode[EventTreeDataType]) -> None:
             self.node: TreeNode[EventTreeDataType] = node
+            """The node that was collapsed."""
             super().__init__()
 
+        @property
+        def control(self) -> Tree[EventTreeDataType]:
+            """The tree that sent the message."""
+            return self.node.tree
+
     class NodeExpanded(Generic[EventTreeDataType], Message, bubble=True):
         """Event sent when a node is expanded.
 
         Can be handled using `on_tree_node_expanded` in a subclass of `Tree` or in a
         parent node in the DOM.
-
-        Attributes:
-            node: The node that was expanded.
         """
 
         def __init__(self, node: TreeNode[EventTreeDataType]) -> None:
             self.node: TreeNode[EventTreeDataType] = node
+            """The node that was expanded."""
             super().__init__()
 
+        @property
+        def control(self) -> Tree[EventTreeDataType]:
+            """The tree that sent the message."""
+            return self.node.tree
+
     class NodeHighlighted(Generic[EventTreeDataType], Message, bubble=True):
         """Event sent when a node is highlighted.
 
         Can be handled using `on_tree_node_highlighted` in a subclass of `Tree` or in a
         parent node in the DOM.
-
-        Attributes:
-            node: The node that was highlighted.
         """
 
         def __init__(self, node: TreeNode[EventTreeDataType]) -> None:
             self.node: TreeNode[EventTreeDataType] = node
+            """The node that was highlighted."""
             super().__init__()
 
+        @property
+        def control(self) -> Tree[EventTreeDataType]:
+            """The tree that sent the message."""
+            return self.node.tree
+
     class NodeSelected(Generic[EventTreeDataType], Message, bubble=True):
         """Event sent when a node is selected.
 
         Can be handled using `on_tree_node_selected` in a subclass of `Tree` or in a
         parent node in the DOM.
-
-        Attributes:
-            node: The node that was selected.
         """
 
         def __init__(self, node: TreeNode[EventTreeDataType]) -> None:
             self.node: TreeNode[EventTreeDataType] = node
+            """The node that was selected."""
             super().__init__()
 
+        @property
+        def control(self) -> Tree[EventTreeDataType]:
+            """The tree that sent the message."""
+            return self.node.tree
+
     def __init__(
         self,
         label: TextType,
@@ -814,6 +875,8 @@ class Tree(Generic[TreeDataType], ScrollView, can_focus=True):
             self._cursor_node = node
             if previous_node != node:
                 self.post_message(self.NodeHighlighted(node))
+        else:
+            self._cursor_node = None
 
     def watch_guide_depth(self, guide_depth: int) -> None:
         self._invalidate()
@@ -976,8 +1039,8 @@ class Tree(Generic[TreeDataType], ScrollView, can_focus=True):
                 "tree--guides-selected", partial=True
             )
 
-            hover = self.root._hover
-            selected = self.root._selected and self.has_focus
+            hover = line.path[0]._hover
+            selected = line.path[0]._selected and self.has_focus
 
             def get_guides(style: Style) -> tuple[str, str, str, str]:
                 """Get the guide strings for a given style.
@@ -1065,10 +1128,8 @@ class Tree(Generic[TreeDataType], ScrollView, can_focus=True):
             return
         if node.is_expanded:
             node.collapse()
-            self.post_message(self.NodeCollapsed(node))
         else:
             node.expand()
-            self.post_message(self.NodeExpanded(node))
 
     async def _on_click(self, event: events.Click) -> None:
         meta = event.style.meta
@@ -1144,7 +1205,7 @@ class Tree(Generic[TreeDataType], ScrollView, can_focus=True):
 
         Note:
             If `auto_expand` is `True` use of this action on a non-leaf node
-            will cause both an expand/collapse event to occour, as well as a
+            will cause both an expand/collapse event to occur, as well as a
             selected event.
         """
         try:
