@@ -99,6 +99,7 @@ WorkType: TypeAlias = Union[
     Callable[[], ResultType],
     Awaitable[ResultType],
 ]
+"""Type used for [workers](/guide/workers/)."""
 
 
 class _ReprText:
@@ -116,11 +117,9 @@ class Worker(Generic[ResultType]):
     """A class to manage concurrent work (either a task or a thread)."""
 
     @rich.repr.auto
-    class StateChanged(Message, bubble=False):
+    class StateChanged(Message, bubble=False, namespace="worker"):
         """The worker state changed."""
 
-        namespace = "worker"
-
         def __init__(self, worker: Worker, state: WorkerState) -> None:
             """Initialize the StateChanged message.
 
@@ -245,6 +244,11 @@ class Worker(Generic[ResultType]):
         """The result of the worker, or `None` if there is no result."""
         return self._result
 
+    @property
+    def error(self) -> BaseException | None:
+        """The exception raised by the worker, or `None` if there was no error."""
+        return self._error
+
     def update(
         self, completed_steps: int | None = None, total_steps: int | None = -1
     ) -> None:
@@ -257,7 +261,7 @@ class Worker(Generic[ResultType]):
         if completed_steps is not None:
             self._completed_steps += completed_steps
         if total_steps != -1:
-            self._total_steps = None if total_steps is None else min(0, total_steps)
+            self._total_steps = None if total_steps is None else max(0, total_steps)
 
     def advance(self, steps: int = 1) -> None:
         """Advance the number of completed steps.
