@@ -1,16 +1,12 @@
 from pathlib import Path
-import sys
 
 import pytest
 
-from textual.widgets import Placeholder
-
 # These paths should be relative to THIS directory.
 WIDGET_EXAMPLES_DIR = Path("../../docs/examples/widgets")
 LAYOUT_EXAMPLES_DIR = Path("../../docs/examples/guide/layout")
 STYLES_EXAMPLES_DIR = Path("../../docs/examples/styles")
 SNAPSHOT_APPS_DIR = Path("./snapshot_apps")
-CLI_PREVIEWS_DIR = Path("../../src/textual/cli/previews")
 
 
 # --- Layout related stuff ---
@@ -78,22 +74,37 @@ def test_switches(snap_compare):
 
 def test_input_and_focus(snap_compare):
     press = [
-        "tab",
-        *"Darren",  # Focus first input, write "Darren"
+        *"Darren",  # Write "Darren"
         "tab",
         *"Burns",  # Focus second input, write "Burns"
     ]
     assert snap_compare(WIDGET_EXAMPLES_DIR / "input.py", press=press)
 
 
+def test_input_validation(snap_compare):
+    """Checking that invalid styling is applied. The snapshot app itself
+    also adds styling for -valid which gives a green border."""
+    press = [
+        *"-2",  # -2 is invalid, so -invalid should be applied
+        "tab",
+        "3",  # This is valid, so -valid should be applied
+        "tab",
+        *"-2",  # -2 is invalid, so -invalid should be applied (and :focus, since we stop here)
+    ]
+    assert snap_compare(SNAPSHOT_APPS_DIR / "input_validation.py", press=press)
+
+
+def test_input_suggestions(snap_compare):
+    assert snap_compare(SNAPSHOT_APPS_DIR / "input_suggestions.py", press=[])
+
+
 def test_buttons_render(snap_compare):
     # Testing button rendering. We press tab to focus the first button too.
-    assert snap_compare(WIDGET_EXAMPLES_DIR / "button.py", press=["tab", "tab"])
+    assert snap_compare(WIDGET_EXAMPLES_DIR / "button.py", press=["tab"])
 
 
 def test_placeholder_render(snap_compare):
     # Testing the rendering of the multiple placeholder variants and labels.
-    Placeholder.reset_color_cycle()
     assert snap_compare(WIDGET_EXAMPLES_DIR / "placeholder.py")
 
 
@@ -132,6 +143,11 @@ def test_datatable_style_ordering(snap_compare):
     assert snap_compare(SNAPSHOT_APPS_DIR / "data_table_style_order.py")
 
 
+def test_datatable_add_column(snap_compare):
+    # Checking adding columns after adding rows
+    assert snap_compare(SNAPSHOT_APPS_DIR / "data_table_add_column.py")
+
+
 def test_footer_render(snap_compare):
     assert snap_compare(WIDGET_EXAMPLES_DIR / "footer.py")
 
@@ -189,7 +205,7 @@ def test_content_switcher_example_initial(snap_compare):
 def test_content_switcher_example_switch(snap_compare):
     assert snap_compare(
         WIDGET_EXAMPLES_DIR / "content_switcher.py",
-        press=["tab", "tab", "enter", "wait:500"],
+        press=["tab", "enter", "wait:500"],
         terminal_size=(50, 50),
     )
 
@@ -198,12 +214,22 @@ def test_tabbed_content(snap_compare):
     assert snap_compare(WIDGET_EXAMPLES_DIR / "tabbed_content.py")
 
 
-def test_option_list(snap_compare):
+def test_option_list_strings(snap_compare):
     assert snap_compare(WIDGET_EXAMPLES_DIR / "option_list_strings.py")
+
+
+def test_option_list_options(snap_compare):
     assert snap_compare(WIDGET_EXAMPLES_DIR / "option_list_options.py")
+
+
+def test_option_list_tables(snap_compare):
     assert snap_compare(WIDGET_EXAMPLES_DIR / "option_list_tables.py")
 
 
+def test_option_list_build(snap_compare):
+    assert snap_compare(SNAPSHOT_APPS_DIR / "option_list.py")
+
+
 def test_progress_bar_indeterminate(snap_compare):
     assert snap_compare(WIDGET_EXAMPLES_DIR / "progress_bar_isolated_.py", press=["f"])
 
@@ -228,6 +254,43 @@ def test_progress_bar_completed_styled(snap_compare):
     assert snap_compare(WIDGET_EXAMPLES_DIR / "progress_bar_styled_.py", press=["u"])
 
 
+def test_select(snap_compare):
+    assert snap_compare(WIDGET_EXAMPLES_DIR / "select_widget.py")
+
+
+def test_selection_list_selected(snap_compare):
+    assert snap_compare(WIDGET_EXAMPLES_DIR / "selection_list_selected.py")
+
+
+def test_selection_list_selections(snap_compare):
+    assert snap_compare(WIDGET_EXAMPLES_DIR / "selection_list_selections.py")
+
+
+def test_selection_list_tuples(snap_compare):
+    assert snap_compare(WIDGET_EXAMPLES_DIR / "selection_list_tuples.py")
+
+
+def test_select_expanded(snap_compare):
+    assert snap_compare(
+        WIDGET_EXAMPLES_DIR / "select_widget.py", press=["tab", "enter"]
+    )
+
+
+def test_select_expanded_changed(snap_compare):
+    assert snap_compare(
+        WIDGET_EXAMPLES_DIR / "select_widget.py",
+        press=["tab", "enter", "down", "enter"],
+    )
+
+
+def test_sparkline_render(snap_compare):
+    assert snap_compare(WIDGET_EXAMPLES_DIR / "sparkline.py")
+
+
+def test_sparkline_component_classes_colors(snap_compare):
+    assert snap_compare(WIDGET_EXAMPLES_DIR / "sparkline_colors.py")
+
+
 # --- CSS properties ---
 # We have a canonical example for each CSS property that is shown in their docs.
 # If any of these change, something has likely broken, so snapshot each of them.
@@ -242,7 +305,6 @@ PATHS = [
 @pytest.mark.parametrize("file_name", PATHS)
 def test_css_property(file_name, snap_compare):
     path_to_app = STYLES_EXAMPLES_DIR / file_name
-    Placeholder.reset_color_cycle()
     assert snap_compare(path_to_app)
 
 
@@ -289,26 +351,6 @@ def test_programmatic_scrollbar_gutter_change(snap_compare):
     )
 
 
-# --- CLI Preview Apps ---
-# For our CLI previews e.g. `textual easing`, `textual colors` etc, we have snapshots
-
-
-def test_borders_preview(snap_compare):
-    assert snap_compare(CLI_PREVIEWS_DIR / "borders.py", press=["tab", "enter"])
-
-
-def test_colors_preview(snap_compare):
-    assert snap_compare(CLI_PREVIEWS_DIR / "colors.py")
-
-
-def test_easing_preview(snap_compare):
-    assert snap_compare(CLI_PREVIEWS_DIR / "easing.py")
-
-
-def test_keys_preview(snap_compare):
-    assert snap_compare(CLI_PREVIEWS_DIR / "keys.py", press=["a", "b"])
-
-
 # --- Other ---
 
 
@@ -358,9 +400,7 @@ def test_disabled_widgets(snap_compare):
 
 
 def test_focus_component_class(snap_compare):
-    assert snap_compare(
-        SNAPSHOT_APPS_DIR / "focus_component_class.py", press=["tab", "tab"]
-    )
+    assert snap_compare(SNAPSHOT_APPS_DIR / "focus_component_class.py", press=["tab"])
 
 
 def test_line_api_scrollbars(snap_compare):
@@ -421,7 +461,7 @@ def test_modal_dialog_bindings_input(snap_compare):
     # Check https://github.com/Textualize/textual/issues/2194
     assert snap_compare(
         SNAPSHOT_APPS_DIR / "modal_screen_bindings.py",
-        press=["enter", "tab", "h", "!", "left", "i", "tab"],
+        press=["enter", "h", "!", "left", "i", "tab"],
     )
 
 
@@ -438,6 +478,27 @@ def test_dock_scroll(snap_compare):
     assert snap_compare(SNAPSHOT_APPS_DIR / "dock_scroll.py", terminal_size=(80, 25))
 
 
+def test_dock_scroll2(snap_compare):
+    # https://github.com/Textualize/textual/issues/2525
+    assert snap_compare(SNAPSHOT_APPS_DIR / "dock_scroll2.py", terminal_size=(80, 25))
+
+
+def test_dock_scroll_off_by_one(snap_compare):
+    # https://github.com/Textualize/textual/issues/2525
+    assert snap_compare(
+        SNAPSHOT_APPS_DIR / "dock_scroll_off_by_one.py",
+        terminal_size=(80, 25),
+        press=["_"],
+    )
+
+
+def test_scroll_to(snap_compare):
+    # https://github.com/Textualize/textual/issues/2525
+    assert snap_compare(
+        SNAPSHOT_APPS_DIR / "scroll_to.py", terminal_size=(80, 25), press=["_"]
+    )
+
+
 def test_auto_fr(snap_compare):
     # https://github.com/Textualize/textual/issues/2220
     assert snap_compare(SNAPSHOT_APPS_DIR / "auto_fr.py", terminal_size=(80, 25))
@@ -453,9 +514,6 @@ def test_scroll_visible(snap_compare):
     assert snap_compare(SNAPSHOT_APPS_DIR / "scroll_visible.py", press=["t"])
 
 
-@pytest.mark.xfail(
-    reason="Unpredictable while https://github.com/Textualize/textual/issues/2254 is open."
-)
 def test_scroll_to_center(snap_compare):
     # READ THIS IF THIS TEST FAILS:
     # While https://github.com/Textualize/textual/issues/2254 is open, the snapshot
@@ -474,3 +532,56 @@ def test_quickly_change_tabs(snap_compare):
 def test_fr_unit_with_min(snap_compare):
     # https://github.com/Textualize/textual/issues/2378
     assert snap_compare(SNAPSHOT_APPS_DIR / "fr_with_min.py")
+
+
+def test_select_rebuild(snap_compare):
+    # https://github.com/Textualize/textual/issues/2557
+    assert snap_compare(
+        SNAPSHOT_APPS_DIR / "select_rebuild.py",
+        press=["space", "escape", "tab", "enter", "tab", "space"],
+    )
+
+
+def test_blur_on_disabled(snap_compare):
+    # https://github.com/Textualize/textual/issues/2641
+    assert snap_compare(
+        SNAPSHOT_APPS_DIR / "blur_on_disabled.py",
+        press=[*"foo", "f3", *"this should not appear"],
+    )
+
+
+def test_tooltips_in_compound_widgets(snap_compare):
+    # https://github.com/Textualize/textual/issues/2641
+    async def run_before(pilot) -> None:
+        await pilot.hover("ProgressBar")
+        await pilot.pause(0.3)
+        await pilot.pause()
+
+    assert snap_compare(SNAPSHOT_APPS_DIR / "tooltips.py", run_before=run_before)
+
+# --- textual-dev library preview tests ---
+
+def test_textual_dev_border_preview(snap_compare):
+    assert snap_compare(SNAPSHOT_APPS_DIR / "dev_previews_border.py", press=["enter"])
+
+
+def test_textual_dev_colors_preview(snap_compare):
+    assert snap_compare(SNAPSHOT_APPS_DIR / "dev_previews_color.py")
+
+
+def test_textual_dev_easing_preview(snap_compare):
+    assert snap_compare(SNAPSHOT_APPS_DIR / "dev_previews_easing.py")
+
+
+def test_textual_dev_keys_preview(snap_compare):
+    assert snap_compare(SNAPSHOT_APPS_DIR / "dev_previews_keys.py", press=["a", "b"])
+
+
+def test_notifications_example(snap_compare) -> None:
+    assert snap_compare(WIDGET_EXAMPLES_DIR / "toast.py")
+
+def test_notifications_through_screens(snap_compare) -> None:
+    assert snap_compare(SNAPSHOT_APPS_DIR / "notification_through_screens.py")
+
+def test_notifications_through_modes(snap_compare) -> None:
+    assert snap_compare(SNAPSHOT_APPS_DIR / "notification_through_modes.py")
