@@ -1,12 +1,14 @@
 import pytest
 from rich.text import Text
 
+from textual import events
 from textual._node_list import DuplicateIds
 from textual.app import App, ComposeResult
 from textual.containers import Container
 from textual.css.errors import StyleValueError
 from textual.css.query import NoMatches
-from textual.geometry import Size
+from textual.geometry import Offset, Size
+from textual.message import Message
 from textual.widget import MountError, PseudoClasses, Widget
 from textual.widgets import Label
 
@@ -260,3 +262,96 @@ def test_render_str() -> None:
     # Text objects are passed unchanged
     text = Text("bar")
     assert widget.render_str(text) is text
+
+
+async def test_compose_order() -> None:
+    from textual.containers import Horizontal
+    from textual.screen import Screen
+    from textual.widgets import Select
+
+    class MyScreen(Screen):
+        def on_mount(self) -> None:
+            self.query_one(Select).value = 1
+
+        def compose(self) -> ComposeResult:
+            yield Horizontal(
+                Select(((str(n), n) for n in range(10)), id="select"),
+                id="screen-horizontal",
+            )
+
+    class SelectBugApp(App[None]):
+        async def on_mount(self):
+            await self.push_screen(MyScreen(id="my-screen"))
+            self.query_one(Select)
+
+    app = SelectBugApp()
+    messages: list[Message] = []
+
+    async with app.run_test(message_hook=messages.append) as pilot:
+        await pilot.pause()
+
+    mounts = [
+        message._sender.id
+        for message in messages
+        if isinstance(message, events.Mount) and message._sender.id is not None
+    ]
+
+    expected = [
+        "_default",  # default  screen
+        "label",  # A static in select
+        "select",  # The select
+        "screen-horizontal",  # The horizontal in MyScreen.compose
+        "my-screen",  # THe screen mounted in the app
+    ]
+
+    assert mounts == expected
+
+
+def test_children_must_be_widgets():
+    with pytest.raises(TypeError):
+        Widget(1, 2, 3)
+
+
+def test_orphan_widget_has_no_siblings():
+    assert Widget().siblings == []
+
+
+def test__allow_scroll_default():
+    assert not Widget()._allow_scroll
+
+
+async def test__allow_scroll():
+    from textual.containers import ScrollableContainer
+
+    class AllowScrollApp(App):
+        CSS = "ScrollableContainer { width: 3; height: 3; }"
+
+        def compose(self):
+            with ScrollableContainer():
+                yield Label("This is\n\n\n\n\nlarge text.")
+
+    app = AllowScrollApp()
+    async with app.run_test():
+        assert app.query_one(ScrollableContainer)._allow_scroll
+
+
+async def test_offset_getter_setter():
+    class OffsetApp(App):
+        def compose(self):
+            yield Label("hello")
+
+    app = OffsetApp()
+    async with app.run_test():
+        label = app.query_one(Label)
+        assert label.offset == Offset(0, 0)
+        label.offset = (7, 3)
+        assert label.offset == Offset(7, 3)
+
+
+def test_get_set_tooltip():
+    widget = Widget()
+    assert widget.tooltip is None
+    widget.tooltip = "This is a tooltip."
+    assert widget.tooltip == "This is a tooltip."
+
+
